@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2022, SERTIT-ICube - France, https://sertit.unistra.fr/
+# Copyright 2023, SERTIT-ICube - France, https://sertit.unistra.fr/
 # This file is part of eoreader project
 #     https://github.com/sertit/eoreader
 #
@@ -42,7 +42,7 @@ from sertit.misc import ListEnum
 from sertit.vectors import WGS84
 from shapely.geometry import Polygon, box
 
-from eoreader import cache, utils
+from eoreader import DATETIME_FMT, EOREADER_NAME, cache, utils
 from eoreader.bands import (
     ALL_CLOUDS,
     BLUE,
@@ -66,7 +66,7 @@ from eoreader.exceptions import InvalidProductError, InvalidTypeError
 from eoreader.products import VhrProduct
 from eoreader.products.optical.optical_product import RawUnits
 from eoreader.reader import Constellation
-from eoreader.utils import DATETIME_FMT, EOREADER_NAME, simplify
+from eoreader.utils import simplify
 
 LOGGER = logging.getLogger(EOREADER_NAME)
 
@@ -974,23 +974,15 @@ class DimapV2Product(VhrProduct):
             # Empty mask cannot be written on file
             mask = gpd.GeoDataFrame(geometry=[], crs=crs)
         else:
-            if self.is_archived:
-                # Open the zip file
-                try:
+            try:
+                if self.is_archived:
+                    # Open the zip file
                     mask = vectors.read(
                         self.path,
                         archive_regex=rf".*MASKS.*{mask_str}.*\.GML",
                         crs=crs,
                     )
-                except Exception:
-                    if mask_str in optional_masks:
-                        mask = gpd.GeoDataFrame(geometry=[], crs=crs)
-                    else:
-                        raise InvalidProductError(
-                            f"Mask {mask_str} not found for {self.path}"
-                        )
-            else:
-                try:
+                else:
                     mask_gml_path = files.get_file_in_dir(
                         self.path.joinpath("MASKS"),
                         f"*{mask_str}*.GML",
@@ -998,13 +990,13 @@ class DimapV2Product(VhrProduct):
                     )
 
                     mask = vectors.read(mask_gml_path, crs=crs)
-                except FileNotFoundError:
-                    if mask_str in optional_masks:
-                        mask = gpd.GeoDataFrame(geometry=[], crs=crs)
-                    else:
-                        raise InvalidProductError(
-                            f"Mask {mask_str} not found for {self.path}"
-                        )
+            except FileNotFoundError:
+                if mask_str in optional_masks:
+                    mask = gpd.GeoDataFrame(geometry=[], crs=crs)
+                else:
+                    raise InvalidProductError(
+                        f"Mask {mask_str} not found for {self.path.joinpath('MASKS')}"
+                    )
 
             # Convert mask to correct CRS
             if not mask.empty and self.product_type in [
@@ -1064,7 +1056,8 @@ class DimapV2Product(VhrProduct):
                 ]
             ):
                 # Convert to target CRS
-                mask.crs = self.crs()
+                mask.crs = self._get_raw_crs()
+                mask = mask.to_crs(self.crs())
 
             # Save to file
             if mask.empty:
