@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2022, SERTIT-ICube - France, https://sertit.unistra.fr/
+# Copyright 2023, SERTIT-ICube - France, https://sertit.unistra.fr/
 # This file is part of eoreader project
 #     https://github.com/sertit/eoreader
 #
@@ -35,29 +35,20 @@ from rasterio.enums import Resampling
 from sertit import files, misc, rasters, snap, strings, vectors
 from sertit.misc import ListEnum
 
-from eoreader import cache, utils
-from eoreader.bands import NEEDED_BANDS, BandNames, SarBand, SarBandMap
+from eoreader import EOREADER_NAME, cache, utils
+from eoreader.bands import BandNames, SarBand, SarBandMap
 from eoreader.bands import SarBandNames as sab
-from eoreader.bands import (
-    compute_index,
-    is_clouds,
-    is_dem,
-    is_index,
-    is_sar_band,
-    is_spectral_band,
-    to_str,
-)
 from eoreader.env_vars import DEM_PATH, DSPK_GRAPH, PP_GRAPH, SAR_DEF_RES, SNAP_DEM_NAME
-from eoreader.exceptions import InvalidBandError, InvalidProductError, InvalidTypeError
+from eoreader.exceptions import InvalidProductError, InvalidTypeError
 from eoreader.keywords import SAR_INTERP_NA
 from eoreader.products.product import Product, SensorType
 from eoreader.reader import Constellation
 from eoreader.stac import INTENSITY
-from eoreader.utils import EOREADER_NAME, simplify
+from eoreader.utils import simplify
 
 LOGGER = logging.getLogger(EOREADER_NAME)
 
-_SAR_PREDICTOR = 1
+SAR_PREDICTOR = 1
 """
 Set LZW predictor to 1 in order SNAP to be able to read this GEoTiff.
 
@@ -637,92 +628,13 @@ class SarProduct(Product):
         # Open bands and get array (resampled if needed)
         band_arrays = {}
         for band_name, band_path in band_paths.items():
-            # Read CSK band
+            # Read SAR band
             band_arrays[band_name] = self._read_band(
                 band_path, resolution=resolution, size=size, **kwargs
             )
 
         return band_arrays
 
-    def _load(
-        self,
-        bands: list,
-        resolution: float = None,
-        size: Union[list, tuple] = None,
-        **kwargs,
-    ) -> dict:
-        """
-        Core function loading SAR data bands
-
-        Args:
-            bands (list): Band list
-            resolution (float): Resolution of the band, in meters
-            size (Union[tuple, list]): Size of the array (width, height). Not used if resolution is provided.
-            kwargs: Other arguments used to load bands
-
-        Returns:
-            Dictionary {band_name, band_xarray}
-        """
-        band_list = []
-        dem_list = []
-        index_list = []
-
-        for band in bands:
-            if is_index(band):
-                if self._has_index(band):
-                    index_list.append(band)
-            elif is_spectral_band(band):
-                raise TypeError(
-                    f"You should ask for SAR bands as {self.name} is a SAR product."
-                )
-            elif is_sar_band(band):
-                if not self.has_band(band):
-                    raise InvalidBandError(
-                        f"{band} cannot be retrieved from {self.condensed_name}"
-                    )
-                else:
-                    band_list.append(band)
-            elif is_dem(band):
-                dem_list.append(band)
-            elif is_clouds(band):
-                raise NotImplementedError(
-                    f"Clouds cannot be retrieved from SAR data ({self.condensed_name})."
-                )
-            else:
-                raise InvalidTypeError(f"{band} is neither a band nor an index !")
-
-        # Check if DEM is set and exists
-        if dem_list:
-            self._check_dem_path(bands, **kwargs)
-
-        # Get all bands to be open
-        bands_to_load = band_list.copy()
-        for idx in index_list:
-            bands_to_load += NEEDED_BANDS[idx]
-
-        # Load band arrays (only keep unique bands: open them only one time !)
-        unique_bands = list(set(bands_to_load))
-        if unique_bands:
-            LOGGER.debug(f"Loading bands {to_str(unique_bands)}")
-        loaded_bands = self._load_bands(
-            unique_bands, resolution=resolution, size=size, **kwargs
-        )
-
-        # Compute index (they conserve the nodata)
-        if index_list:
-            LOGGER.debug(f"Loading indices {to_str(index_list)}")
-        bands_dict = {idx: compute_index(idx, loaded_bands) for idx in index_list}
-
-        # Add bands
-        bands_dict.update({band: loaded_bands[band] for band in band_list})
-
-        # Add DEM
-        bands_dict.update(
-            self._load_dem(dem_list, resolution=resolution, size=size, **kwargs)
-        )
-
-        return bands_dict
-
     def _pre_process_sar(self, band: sab, resolution: float = None, **kwargs) -> str:
         """
         Pre-process SAR data (geocoding...)
@@ -753,7 +665,7 @@ class SarProduct(Product):
                     file_path,
                     dtype=np.float32,
                     nodata=self._snap_no_data,
-                    predictor=_SAR_PREDICTOR,
+                    predictor=SAR_PREDICTOR,
                 )
             return file_path
         else:
@@ -765,15 +677,38 @@ class SarProduct(Product):
 
                 # Pre-process graph
                 if PP_GRAPH not in os.environ:
-                    sat = (
-                        "s1"
-                        if self.constellation_id == Constellation.S1.name
-                        else "sar"
-                    )
-                    spt = "grd" if self.sar_prod_type == SarProductType.GDRG else "cplx"
-                    pp_graph = utils.get_data_dir().joinpath(
-                        f"{spt}_{sat}_preprocess_default.xml"
-                    )
+                    if self.constellation == Constellation.CAPELLA:
+                        LOGGER.debug(
+                            "SNAP Error: [NodeId: Calibration] Mission Capella is currently not supported for calibration. Removing this step."
+                        )
+                        pp_graph = utils.get_data_dir().joinpath(
+                            "grd_sar_preprocess_fallback.xml"
+                        )
+                    elif (
+                        self.constellation == Constellation.CSG
+                        and self.sar_prod_type == SarProductType.CPLX
+                    ):
+                        LOGGER.debug(
+                            "SNAP Error: Calibration currently fails for CSG complex data. Removing this step."
+                        )
+                        pp_graph = utils.get_data_dir().joinpath(
+                            "cplx_no_calib_preprocess_default.xml"
+                        )
+
+                    else:
+                        sat = (
+                            "s1"
+                            if self.constellation_id == Constellation.S1.name
+                            else "sar"
+                        )
+                        spt = (
+                            "grd"
+                            if self.sar_prod_type == SarProductType.GDRG
+                            else "cplx"
+                        )
+                        pp_graph = utils.get_data_dir().joinpath(
+                            f"{spt}_{sat}_preprocess_default.xml"
+                        )
                 else:
                     pp_graph = AnyPath(os.environ[PP_GRAPH])
                     if not pp_graph.is_file() or not pp_graph.suffix == ".xml":
@@ -811,21 +746,25 @@ class SarProduct(Product):
                         dem_path = ""
 
                     # Download cloud path to cache
-                    if isinstance(self.path, CloudPath):
-                        LOGGER.debug(
-                            f"Caching {self.path} to {os.path.join(tmp_dir, self.path.name)}"
-                        )
-                        if self.path.is_dir():
-                            prod_path = os.path.join(
-                                tmp_dir, self.path.name, self.snap_filename
+                    prod_path = kwargs.get("prod_path")
+                    if prod_path is None:
+                        if isinstance(self.path, CloudPath):
+                            LOGGER.debug(
+                                f"Caching {self.path} to {os.path.join(tmp_dir, self.path.name)}"
                             )
-                            self.path.download_to(os.path.join(tmp_dir, self.path.name))
+                            if self.path.is_dir():
+                                prod_path = os.path.join(
+                                    tmp_dir, self.path.name, self.snap_filename
+                                )
+                                self.path.download_to(
+                                    os.path.join(tmp_dir, self.path.name)
+                                )
+                            else:
+                                prod_path = (
+                                    self.path.fspath
+                                )  # In tmp file, no need to download_to
                         else:
-                            prod_path = (
-                                self.path.fspath
-                            )  # In tmp file, no need to download_to
-                    else:
-                        prod_path = self.path.joinpath(self.snap_filename)
+                            prod_path = self.path.joinpath(self.snap_filename)
 
                     # Create SNAP CLI
                     cmd_list = snap.get_gpt_cli(
@@ -945,10 +884,13 @@ class SarProduct(Product):
                 arr = interp_na(arr, dim="y")
                 arr = interp_na(arr, dim="x")
 
+            # Get suffix
+            suffix = kwargs.get("suffix")
+
             # Save the file as the terrain-corrected image
             file_path = os.path.join(
                 self._get_band_folder(writable=True),
-                f"{files.get_filename(dim_path)}_{pol}{'_DSPK' if dspk else ''}.tif",
+                f"{files.get_filename(dim_path)}_{pol}{'_DSPK' if dspk else ''}{f'_{suffix}' if suffix else ''}.tif",
             )
             # WARNING: Set nodata to 0 here as it is the value wanted by SNAP !
 
@@ -960,7 +902,7 @@ class SarProduct(Product):
                 file_path,
                 dtype=np.float32,
                 nodata=self._snap_no_data,
-                predictor=_SAR_PREDICTOR,
+                predictor=SAR_PREDICTOR,
             )
 
         return file_path
@@ -1019,8 +961,9 @@ class SarProduct(Product):
         Args:
             xarr (xr.DataArray): Array whose attributes need an update
             bands (list): Array name (as a str or a list)
+
         Returns:
-            xr.DataArray: Updated array
+            xr.DataArray: Updated array/dataset
         """
 
         return xarr
