@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2022, SERTIT-ICube - France, https://sertit.unistra.fr/
+# Copyright 2023, SERTIT-ICube - France, https://sertit.unistra.fr/
 # This file is part of eoreader project
 #     https://github.com/sertit/eoreader
 #
@@ -30,19 +30,19 @@ import xarray as xr
 from cloudpathlib import AnyPath, CloudPath
 from lxml import etree
 from rasterio import errors
-from rasterio.control import GroundControlPoint
 from rasterio.enums import Resampling
 from rasterio.errors import NotGeoreferencedWarning
 from rasterio.rpc import RPC
 from sertit import rasters
 
+from eoreader import EOREADER_NAME
+from eoreader.bands import is_index, is_sat_band, to_str
 from eoreader.env_vars import USE_DASK
 from eoreader.exceptions import InvalidProductError
 from eoreader.keywords import _prune_keywords
 
-EOREADER_NAME = "eoreader"
-DATETIME_FMT = "%Y%m%dT%H%M%S"
 LOGGER = logging.getLogger(EOREADER_NAME)
+UINT16_NODATA = 65535
 
 
 def get_src_dir() -> Union[CloudPath, Path]:
@@ -157,6 +157,8 @@ def read(
         xr.DataArray: Masked xarray corresponding to the raster data and its metadata
 
     """
+    window = kwargs.get("window")
+
     if use_dask():
         chunks = True
     else:
@@ -169,15 +171,16 @@ def read(
             return rasters.read(
                 path,
                 resolution=resolution,
-                size=size,
                 resampling=resampling,
                 masked=masked,
                 indexes=indexes,
+                size=size if window is None else None,
+                window=window,
                 chunks=chunks,
-                **_prune_keywords(**kwargs),
+                **_prune_keywords(additional_keywords=["window"], **kwargs),
             )
     except errors.RasterioIOError as ex:
-        if str(path).endswith("jp2") or str(path).endswith("tif"):
+        if (str(path).endswith("jp2") or str(path).endswith("tif")) and path.exists():
             raise InvalidProductError(f"Corrupted file: {path}") from ex
         else:
             raise
@@ -222,60 +225,13 @@ def write(xds: xr.DataArray, path: Union[str, CloudPath, Path], **kwargs) -> Non
             pass
 
     # Write
-    rasters.write(xds, path=path, lock=lock, **_prune_keywords(**kwargs))
+    rasters.write(xds, path=path, lock=lock, **_prune_keywords(["window"], **kwargs))
 
     # Set back the previous long name
     if previous_long_name and xds.rio.count > 1:
         xds.attrs["long_name"] = previous_long_name
 
 
-def create_gcps(lon: xr.DataArray, lat: xr.DataArray, alt: xr.DataArray) -> list:
-    """
-    Create GCPs from an array of longitude, latitude and altitude (based on Sentinel-3 geocoding).
-
-    Args:
-        lon (xr.DataArray): Longitude array
-        lat (xr.DataArray): Latitude array
-        alt (xr.DataArray): Altitude array
-
-    Returns:
-        list: List of GroundControlPoints
-
-    """
-    gcps = []
-    assert lat.data.shape == lon.data.shape == alt.data.shape
-
-    # Get the GCPs coordinates
-    nof_gcp_x = np.linspace(0, lat.rio.width - 1, dtype=int)
-    nof_gcp_y = np.linspace(0, lat.rio.height - 1, dtype=int)
-
-    # Create the GCP sequence
-    gcp_id = 0
-    for x in nof_gcp_x:
-        for y in nof_gcp_y:
-            curr_lon = lon.data[0, y, x]
-            curr_lat = lat.data[0, y, x]
-            curr_alt = alt.data[0, y, x]
-            if (
-                not np.isnan(curr_lon)
-                and not np.isnan(curr_lat)
-                and not np.isnan(curr_alt)
-            ):
-                gcps.append(
-                    GroundControlPoint(
-                        row=y,
-                        col=x,
-                        x=lon.data[0, y, x],
-                        y=lat.data[0, y, x],
-                        z=alt.data[0, y, x],
-                        id=gcp_id,
-                    )
-                )
-                gcp_id += 1
-
-    return gcps
-
-
 def quick_xml_to_dict(element: etree._Element) -> tuple:
     """
     Convert a lxml root to a nested dict (quick and dirty)
@@ -421,3 +377,84 @@ def simplify(footprint_fct: Callable):
         return simplify_footprint(footprint, self.resolution)
 
     return simplify_wrapper
+
+
+def stack_dict(
+    bands: list, band_dict: dict, save_as_int: bool, nodata: float, **kwargs
+) -> (xr.DataArray, type):
+    """
+    Stack a dictionnary containing bands in a DataArray
+
+    Args:
+        bands (list): List of bands (to keep the right order of the stack)
+        band_dict (dict): Dict containing the bands as xr.DataArray {band_name, band}
+        save_as_int (bool): Convert stack to uint16 to save disk space (and therefore multiply the values by 10.000)
+        nodata (float): Nodata value
+
+    Returns:
+        (xr.DataArray, type): Stack as a DataArray and its dtype
+    """
+    # Convert into dataset with str as names
+    LOGGER.debug("Stacking")
+    data_vars = {}
+    coords = band_dict[bands[0]].coords
+    for key in bands:
+        data_vars[to_str(key)[0]] = (
+            band_dict[key].coords.dims,
+            band_dict[key].data,
+        )
+
+        # Set memory free (for big stacks)
+        band_dict[key].close()
+        band_dict[key] = None
+
+    # Create dataset, with dims well-ordered
+    stack = (
+        xr.Dataset(
+            data_vars=data_vars,
+            coords=coords,
+        )
+        .to_stacked_array(new_dim="z", sample_dims=("x", "y"))
+        .transpose("z", "y", "x")
+    )
+
+    # Save as integer
+    dtype = np.float32
+    if save_as_int:
+        scale = 10000
+        stack_min = np.nanmin(stack.data)
+        if np.round(stack_min * 1000) / 1000 < -0.1:
+            LOGGER.warning(
+                f"Cannot convert the stack to uint16 as it has negative values ({stack_min} < -0.1). Keeping it in float32."
+            )
+        else:
+            if stack_min < 0:
+                LOGGER.warning(
+                    "Small negative values ]-0.1, 0] have been found. Clipping to 0."
+                )
+                stack = stack.copy(data=np.clip(stack.data, a_min=0, a_max=None))
+
+            # Scale to uint16, fill nan and convert to uint16
+            dtype = np.uint16
+            for b_id, band in enumerate(bands):
+                # SCALING
+                # NOT ALL bands need to be scaled, only:
+                # - Satellite bands
+                # - index
+                if is_sat_band(band) or is_index(band):
+                    if np.nanmax(stack[b_id, ...]) > UINT16_NODATA / scale:
+                        LOGGER.debug(
+                            "Band not in reflectance, keeping them as is (the values will be rounded)"
+                        )
+                    else:
+                        stack[b_id, ...] = stack[b_id, ...] * scale
+
+                # Fill no data (done here to avoid RAM saturation)
+                stack[b_id, ...] = stack[b_id, ...].fillna(nodata)
+
+    if dtype == np.float32:
+        # Set nodata if needed (NaN values are already set)
+        if stack.rio.encoded_nodata != nodata:
+            stack = stack.rio.write_nodata(nodata, encoded=True, inplace=True)
+
+    return stack, dtype
